Ext.define('App.controller.Viewport', {
    extend: 'Ext.app.Controller',
    controllers: [
        'Report',
        'Navigation',
        'viewer.Chart'
    ],
    views: [
        'viewport.error.ErrorConnection',
        'viewport.error.ErrorCritical',
        'viewport.Button',
        'viewport.PickerButton',
        'viewport.Switcher',
        'viewport.LinkSwitcher',

        'navigation.Settings',
        'navigation.Informer',
        
        'reports.picker.Selection',
        'reports.Picker',
        'reports.Save',
        'Reports',

        'filter.Field',
        'filter.field.combo.list.SelectionScroller',
        'filter.field.combo.list.SelectionModel',
        'filter.field.combo.list.SelectionView',
        'filter.field.combo.AbstractList',
        'filter.field.combo.AbstractPicker',
        'filter.field.combo.List',
        'filter.field.combo.Picker',
        'filter.field.combo.Search',
        'filter.field.combo.SimpleList',
        'filter.field.combo.SimplePicker',
        'filter.field.Combo',
        'filter.field.Destination',
        'filter.field.DestinationExt',
        'filter.field.DestinationInt',
        'filter.field.Trunk',
        'filter.field.Tag',
        'filter.field.Pbx',
        'filter.minMaxEditor.Spinner',
        'filter.Observer',
        'filter.Editor',
        'filter.MinMaxEditor',
        'filter.DestinationField',
        'filter.DestinationEditor',
        'filter.TypesField',
        'filter.TypesEditor',
        'filter.CostField',
        'filter.CostEditor',
        'filter.PeriodField',
        'filter.PeriodEditor',
        'filter.BillField',
        'filter.BillEditor',
        'filter.PbxesField',
        'filter.PbxesEditor',
        'filter.TrunksField',
        'filter.TrunksEditor',
        'filter.TagsField',
        'filter.TagsEditor',
        
        'grouping.Picker',
        'metric.Picker',

        'viewer.FullPage',
        'viewer.ExportButton',
        'viewer.ExportDialog',
        'viewer.Metric',
        'viewer.Grouping',

        'viewer.chart.series.Pie',
        'viewer.chart.ChartView',
        'viewer.Chart',
        'viewer.table.Column',
        'viewer.table.Chunker',
        'viewer.table.Scroller',
        'viewer.table.Summary',
        'viewer.table.View',
        'viewer.Table',
        'viewer.error.EmptyResults',
        'viewer.Error',
        'Filter',
        'Viewer',
        'Navigation',
        'Viewport'
    ],
    models: [
        'SimpleEvent',
        'GroupedEvent',
        'Filter',
        'Destination',
        'DestinationInt',
        'DestinationExt',
        'Trunk',
        'Tag',
        'Pbx'
    ],
    stores: [
        'SimpleEvents',
        'GroupedEvents',
        'Filters',
        'DestinationsInt',
        'DestinationsExt',
        'Pbxes',
        'Trunks',
        'Tags'
    ],
    requires: [
        'App.proxy.AbstractProxy',
        'App.proxy.AbstractEvents',
        'App.proxy.SimpleEvents',
        'App.proxy.GroupedEvents',
        'App.proxy.Destinations',
        'App.proxy.DestinationsInt',
        'App.proxy.DestinationsExt',
        'App.proxy.Trunks',
        'App.proxy.Tags',
        'App.proxy.Pbxes',
        'App.proxy.Filters',
        'App.store.AbstractStore',
        'App.store.Destinations'
    ],
    init: function() {
        var self = this;

        this.addEvents('exceptionError', 'connectionError', 'connectionRestored');

        this.on('exceptionError', this.onExceptionError, this);
        this.on('connectionError', this.onConnectionError, this);
        this.on('connectionRestored', this.onConnectionRestored, this);

        // Bind failure responses for synchronizer
        this.synchronizer = App.storage.Synchronizer;
        this.synchronizer.on('exceptionError', this.onExceptionError, this);
        this.synchronizer.on('connectionError', this.onConnectionError, this);
        this.synchronizer.on('connectionRestored', this.onConnectionRestored, this);

        this.view = Ext.create('App.view.Viewport', { renderTo: Ext.getBody() } );
        this.callParent(arguments);

        this.interval = 10 * 60 * 1000;
        setTimeout(function() { self.onSyncEventsAuto.call(self) }, self.interval);
    },

    // Methods for automatic synchronization

    onSyncEventsAuto: function() {
        var self = this;

        if (this.synchronizer.isWorking()) {
            setTimeout(function() { self.onSyncEventsAuto.call(self) }, self.interval);
            return;
        }

        this.connErrorSilent = true;
        this.bindSynchronizer();
        this.synchronizer.fireEvent('syncEventsAuto');
    },

    onSyncEventsCompleted: function() {
        console.log('viewport', 'onSyncEventsCompleted');

        var self = this;

        this.connErrorSilent = false;
        this.unbindSynchronizer();
        this.getController('Navigation').fireEvent('redrawInformer');
        setTimeout(function() { self.onSyncEventsAuto.call(self) }, self.interval);
    },

    onSyncEventsTerminated: function() {
        console.log('viewport', 'onSyncEventsTerminated');

        var self = this;

        this.connErrorSilent = false;
        this.unbindSynchronizer();
        this.getController('Navigation').fireEvent('redrawInformer');
        setTimeout(function() { self.onSyncEventsAuto.call(self) }, 10000);
    },


    bindSynchronizer: function() {
        this.synchronizer.on('syncEventsComplete', this.onSyncEventsCompleted, this);
        this.synchronizer.on('syncTerminated', this.onSyncEventsTerminated, this);
    },

    unbindSynchronizer: function() {
        this.synchronizer.un('syncEventsComplete', this.onSyncEventsCompleted, this);
        this.synchronizer.un('syncTerminated', this.onSyncEventsTerminated, this);
    },

    // Server response methods

    /**
     * Fires when we found an exception that can be handled, we should show a message with problem
     * and possibility to continue work or refresh a page.
     */
    onExceptionError: function(message) {
        if (this.connErrorDialog) {
            this.connErrorDialog.hide();
        }

        this.errorDialog = Ext.create('App.view.viewport.error.ErrorCritical', {
            response: message
        });
        this.errorDialog.show();
    },

    /**
     * Fires when we found a connection problem, we should show notification
     * about this at top of page.
     */
    onConnectionError: function(callback, scope) {
        if (!this.connErrorDialog) {
            this.connErrorDialog = Ext.create('App.view.viewport.error.ErrorConnection');
        }

        this.connProblems = true;
        this.connErrorCallback = callback;
        this.connErrorScope = scope || this;

        this.connErrorDialog.handle(30, this.onConnectionErrorCallback, this, this.connErrorSilent);
    },

    /**
     * Fires when connection to pbx was successfully restored, we should show a notification
     * about connection problems.
     */
    onConnectionRestored: function() {
        if (this.connErrorDialog) {
            this.connErrorDialog.hide();
        }
    },

    /**
     * Fires when we should repeat failure request to a server
     */
    onConnectionErrorCallback: function() {
        this.connErrorCallback.call(this.connErrorScope);
    },


    /**
     * Process a response from the server, if its not valid we should show a needed notification.
     */
    processResponse: function(text, callbackSuccess, callbackRetry, scope) {
        var result;
        var timeout;
        var exception = _t('Unknown error');

        scope = scope || this;

        try {
            result = Ext.decode(text);
        } catch (e) {
            // Server returns an unsupported format
            this.fireEvent('exceptionError', text);
            return;
        }

        timeout = parseInt(result['timeout']);

        if (result.hasOwnProperty('skipped') && parseInt(result['skipped']) === 1) {
            // Request was skipped by a server
            if (result.hasOwnProperty('exception')) {
                exception = result['exception'];
            }

            if (exception == 'TRY_AGAIN_LATER') {
                // Request was correctly skipped by a server and we should wait for a timeout and fire Retry callback
                if (this.attempts === undefined) {
                    this.attempts = 0;
                }

                if (this.attempts >= 5) {
                    setTimeout(function() { callbackRetry.call(scope, true); }, timeout * 1000);
                } else {
                    setTimeout(function() { callbackRetry.call(scope, false); }, timeout * 1000);
                }

                this.attempts++;
                return;
            }

            // Unknow exception we should stop any execution and show an error
            this.fireEvent('exceptionError', exception);
            return;
        }

        if (this.connProblems) {
            this.connProblems = false;
            this.fireEvent('connectionRestored');
        }

        // Normal request
        callbackSuccess.call(scope, result, text);
    }
} );
