Ext.define('App.controller.viewer.Table', {
    extend: 'Ext.app.Controller',
    refs: [
        {
            ref: 'viewerCnt',
            selector: 'viewer'
        },
        {
            ref: 'tablePanel',
            selector: 'viewerTable'
        }
    ],
    init: function() {
        var self = this;

        this.type = 1;
        this.reportId = 0;
        this.simpleStore = Ext.data.StoreManager.lookup('SimpleEvents');
        this.groupedStore = Ext.data.StoreManager.lookup('GroupedEvents');

        this.simpleStore.on('guaranteedrange', function() {
            self.onStoreGuaranteedRange.call(self, self.simpleStore);
        }, this);
        this.groupedStore.on('guaranteedrange', function() {
            self.onStoreGuaranteedRange.call(self, self.groupedStore);
        }, this);

        this.callParent(arguments);
    },

    setSorters: function(sorters){
        if (this.type == 1)  {
           this.simpleStore.setBuilderSorters(sorters);
        } else if (this.type == 2) {
            this.groupedStore.setBuilderSorters(sorters);
        }
    },
    setState: function(state) {
        this.type = state.type;
        this.setSorters(state.sorters);
    },

    getState: function() {
        var state = new Object();
        state['type'] = this.type;
        state['sorters'] = this.getCurrentSorters();
        return state;
    },

    getCurrentSorters: function(){
        if (this.type == 1)  {
           return this.simpleStore.getBuilderSorters();
        } else if (this.type == 2) {
            return this.groupedStore.getBuilderSorters();
        }
        return undefined;
    },

    getCurrentStore: function() {
        if (this.type == 1)  {
           return this.simpleStore;
        } else if (this.type == 2) {
            return this.groupedStore;
        }
        return undefined;
    },

    /**
     * Methods that ...
     */
    report: function(grouping, metric, filter) {
        var groupingChange = false;

        // TODO: More simple way from Report contoroller with additional arguments to report filterChange & groupingChange
        if (this.previosGrouping) {
            if (this.previosGrouping.length != grouping.length) {
                groupingChange = true;
            } else {
                var i=0;

                for (i=0; i < grouping.length; i++) {
                    if (this.previosGrouping[i] != grouping[i]) {
                        groupingChange = true;
                        break;
                    }
                }
            }
        }

        this.previosFilter = filter;
        this.previosGrouping = grouping;

        if (grouping.length == 0 ) {
            this.type = 1;
        } else {
            this.type = 2;
        }

        if (this.type == 1) {
            return this.reportSimple(filter);
        } else {
            return this.reportGrouped(grouping, metric, filter);
        }
    },

    reportPlainWithCurrentConditions: function(callback, scope) {
        this.plainReportStore = this.getCurrentStore();
        this.plainReportTotal = this.plainReportStore.getTotalCount();
        this.plainReportRows = [];
        this.plainReportCallback = callback;
        this.plainReportScope = scope;

        if (this.plainReportTotal > 500000) {
            this.plainReportTotal = 500000;
        }

        var step = 100000;
        var total = this.plainReportTotal;
        var seq = Promise.resolve();
        var self = this;

        for (var start = 0; start < total; start += step) {
            (function (start) {
                seq = seq.then(function(){
                    return self.loadEvents(start, step);
                });
            })(start);
        }
    },

    loadEvents: function(start, step) {
        var store = this.plainReportStore;
        var self = this;
        var operation = new Ext.data.Operation();
        operation.start = start;
        operation.limit = step;
        if (start + step > this.plainReportTotal) {
            operation.limit = this.plainReportTotal - start;
        }

        return new Promise(function(resolve){
            store.getProxy()[self.type==1?'readSimple':'readGrouped'](operation, function(result){
                self.onReportPlainDataFetched(result);
                resolve(result);
            });
        });
    },

    onReportPlainDataFetched: function(result) {
        this.plainReportRows = this.plainReportRows.concat(result.getRecords());
        console.log("this.plainReportRows.length", this.plainReportRows.length);
        console.log("this.plainReportTotal", this.plainReportTotal);

        if (this.plainReportRows.length >= this.plainReportTotal) {
            var rows = this.plainReportRows;
            var store = this.plainReportStore;
            var columns = this.getTablePanel().columns;

            // Data is ready we should fire needed event
            console.log("getBuilderFilters", store.getBuilderFilters());
            console.log("getBuilderColumns", store.getBuilderColumns());
            console.log("getBuilderGroupings", store.getBuilderGroupings());
            console.log("getBuilderSorters", store.getBuilderSorters());
            console.log("getBuilderSummaries", store.getBuilderSummaries());

            result = {
                headers: [],
                rows: [],
                summaries: []
            };
            var renderers = [];

            // Generate headers and summaries
            for (var i = 0; i < columns.length; i++) {
                var column = columns[i];

                if (column.identy === 'NUMBERER' || column.hidden) {
                    continue;
                }

                result.headers.push(column.text);

                if (Ext.isFunction(column.exportRenderer)) {
                    renderers.push({index: column.dataIndex, renderer: column.exportRenderer});
                } else if (Ext.isFunction(column.render)) {
                    renderers.push({index: column.dataIndex, renderer: column.renderer, skipTags: true});
                } else {
                    renderers.push({index: column.dataIndex, renderer: function(v) { return v; }});
                }

                if (Ext.isFunction(column.summaryRenderer)) {
                    this.plainReportStore.summaries.skipTags = true;
                    result.summaries.push(column.summaryRenderer(this.plainReportStore.summaries));
                } else {
                    result.summaries.push("");
                }
            }

            var tempRow = {};

            // Generate rows
            for (var c = 0; c < rows.length; c++) {
                var row = new Array(renderers.length);
                for (var r = 0; r < renderers.length; r++) {
                    if (rows[c]['data'].hasOwnProperty(renderers[r].index)) {
                        row[r] = renderers[r].renderer(rows[c]['data'][renderers[r].index], tempRow, rows[c]);
                    }
                }

                result.rows.push(row);
            }

            this.plainReportRows = [];
            var callback = this.plainReportCallback;
            var scope = this.plainReportScope ? this.plainReportScope : this;

            callback.call(scope, result);
        }
    },

    reportSimple: function(filter) {
        console.log('reportSimple');
        this.reportId++;

        var columns = this.getSimpleMetricColumns();
        var builderColumns = this.getBuilderColumnsByIdenties(columns);
        var builderSummaries = this.getBuilderSummariesByIdenties(columns);
        var builderSorters = this.simpleStore.getBuilderSorters() ? this.simpleStore.getBuilderSorters() : this.getBuilderDefaultSorters();

        // Adding default columns
        columns.reverse();
        columns.push('NUMBERER');
        columns.reverse();

        builderSummaries.push( {name: 'COLUMN_TOTAL_EVENTS', as: 'TOTAL_EVENTS'} );

        // Modify store condition
        this.simpleStore.setReportId(this.reportId);
        this.simpleStore.setBuilderFilters(filter);
        this.simpleStore.setBuilderColumns(builderColumns);
        this.simpleStore.setBuilderSummaries(builderSummaries);
        this.simpleStore.setBuilderSorters(builderSorters);

        // Clear store results
        this.getTablePanel().clearStore();

        // Clear view results
        this.getTablePanel().clearView();

        // Modify table columns
        this.getTablePanel().clearColumns(columns, 'simple', this.getCurrentSorters());

        // Refresh store
        this.simpleStore.guaranteeRange(0, this.simpleStore.pageSize - 1);
    },

    reportGrouped: function(grouping, metric, filter) {
        this.reportId++;

        var columns = this.getGroupedMetricColumns(grouping, metric);
        var builderColumns = this.getBuilderColumnsByIdenties(columns);
        var builderGroupings = this.getBuilderGroupingsByIdenties(columns);
        var builderSorters = this.getBuilderGroupingsSortersByIdenties(columns);
        var builderSummaries = this.getBuilderSummariesByIdenties(columns);
        var builderSummariesType = 'simple';

        // Define summaries type
        var i;

        for (i=0; i < columns.length; i++) {
            if (columns[i] == "GROUP_USER_COMM_FROM" || columns[i] == "GROUP_USER_COMM_TO" || columns[i] == "GROUP_GROUP_COMM_FROM" || columns[i] == "GROUP_GROUP_COMM_TO") {
                builderSummariesType = 'extended';
                break;
            }
        }


        // Modify store condition
        this.groupedStore.setReportId(this.reportId);
        this.groupedStore.setBuilderFilters(filter);
        this.groupedStore.setBuilderColumns(builderColumns);
        this.groupedStore.setBuilderGroupings(builderGroupings);
        this.groupedStore.setBuilderSorters(builderSorters);
        this.groupedStore.setBuilderSummaries(builderSummaries);
        this.groupedStore.setBuilderSummariesType(builderSummariesType);

        // Clear store results
        this.getTablePanel().clearStore();

        // Clear view results
        this.getTablePanel().clearView();

        // Modify table columns
        this.getTablePanel().clearColumns(columns, 'grouped');

        // Refresh store
        this.groupedStore.guaranteeRange(0, this.groupedStore.pageSize - 1);
    },

    /**
     * Fires when user change a columns
     */
    onTableColumnsChange: function(columns) {
        if (this.type == 1) {
            return this.reportSimple(this.previosFilter, columns);
        } else {
            return this.reportGrouped(this.previosFilter, this.previosGrouping, columns);
        }
    },
    /**
     * Fires when simple or grouped store was loaded,
     * we should verify for empty results and show notofication.
     */
    onStoreGuaranteedRange: function(store) {
        var viewer;

        if (store.getTotalCount() < 1) {
            viewer = this.getController('Viewer');
            viewer.setErrorState('ERR_EMPTYRESULTS');
            return;
        } else {
            viewer = this.getController('Viewer');
            viewer.setExportButtonEnabled(true);
        }
    },

    getBuilderColumnsByIdenties: function(identies) {
        return this.getViewerController().getBuilderColumnsByIdenties(identies);
    },

    getBuilderGroupingsByIdenties: function(identies) {
        return this.getViewerController().getBuilderGroupingsByIdenties(identies);
    },

    getBuilderDefaultSorters: function() {
        return this.getViewerController().getBuilderDefaultSorters();
    },

    getBuilderGroupingsSortersByIdenties: function(identies) {
        return this.getViewerController().getBuilderGroupingsSortersByIdenties(identies);
    },

    getBuilderSummariesByIdenties: function(identies) {
        return this.getViewerController().getBuilderSummariesByIdenties(identies);
    },

    getGroupedMetricColumns: function(grouping, metric) {
        return this.getViewerController().getGroupedMetricColumns(grouping, metric);
    },

    getSimpleMetricColumns: function() {
        return this.getViewerController().getSimpleMetricColumns();
    },

    getViewerController: function() {
        return this.getController("Viewer");
    }
} );
