Ext.define('App.view.bootstrap.Sync', {
    extend: 'App.view.bootstrap.Notification',
    title: '',
    width: 394,
    initComponent: function() {
        this.addEvents(
            'сancelButtonClick'
        );
        this.callParent(arguments);
    },
    afterRender: function() {
        this.callParent(arguments);
        
        this.descriptionEl = Ext.create('Ext.container.Container', {
            renderTo: this.formEl,
            cls: 'w-boot-description'
        });
        
        this.progressActiveEl = Ext.create('Ext.container.Container', {
            cls: 'w-boot-progress-active',
            width: 25
        });
        
        this.progressDescriptionEl = Ext.create('Ext.container.Container', {
            cls: 'w-boot-progress-description'
        });
        
        this.progressEl = Ext.create('Ext.container.Container', {
            cls: 'w-boot-progress',
            items: [
                this.progressActiveEl
            ]
        });

        this.cancelButton = Ext.create('Ext.button.Button', {
            text: _t('Cancel'),
            disabled: true
        });

        this.cancelEl = Ext.create('Ext.container.Container', {
            cls: 'w-boot-cancel',
//            hidden: true,
            items: [
                this.cancelButton
            ]
        });

        this.progressContainerEl = Ext.create('Ext.container.Container', {
            xtype: 'container',
            renderTo: this.formEl,
            items: [
                {
                    xtype: 'container',
                    layout: 'hbox',
                    padding: '0 0 0 0',
                    items: [
                        {
                            xtype: 'container',
                            width: 24,
                            cls: 'w-boot-loader'
                        },
                        {
                            xtype: 'container',
                            width: 340,
                            padding: '0 0 0 10',
                            items: [
                                this.progressEl,
                                this.progressDescriptionEl
                            ]
                        }
                    ]
                },
                {
                    xtype: 'container',
                    height: 32,
                    padding: '10 0 0 0',
                    layout: {
                        type: 'vbox',
                        align: 'center'
                    },
                    items: [
                        this.cancelEl
                    ]
                }
            ]
        });
        this.cancelButton.on('click', this.onCancelButtonClick, this);
    },
    setProgress: function(percent) {
        var width = (330 / 100) * percent;
        var animation = {
            to: {
                width: width
            }
        }
        
        this.progressActiveEl.stopAnimation();
        this.progressActiveEl.animate(animation);
    },
    setRelateProgress: function(percent, offset, percentTo) {
        var area = percentTo - offset;
        var point = area / percentTo;
        var total = offset + Math.ceil(percent * point);
        
        if (total > percentTo) {
            total = percentTo;
        }
        
        this.setProgress(total);
    },
    setDescription: function(text) {
        this.progressDescriptionEl.getEl().update(text);
    },
    markUsersSync: function(percent, timeleft) {
        if (!percent) {
            this.setProgress(12);
            this.setDescription(_t('Loading users...'));
            return;
        }

        if (percent > 0) {
            this.setRelateProgress(percent, 12, 44);
        }

        if (timeleft<0) timeleft = 0;
        timeleft = App.utility.NamesFormatter.formatSeconds(Math.round(timeleft));

        this.setDescription(_t('Loading users...') +  sprintf(_t('(remaining ~ %(timeleft)s)'), { timeleft : timeleft }));
    },
    markExtraSync: function() {
        this.setProgress(44);
        this.setDescription(_t('Loading groups, trunks, pbxes...'));
    },
    markBackendSync: function() {
        this.setProgress(49);
        this.setDescription(_t('Loading backend...'));
    },
    markEventsSync: function(percent, timeleft) {
        this.cancelButton.enable();
        if (!percent && !timeleft) {
            this.setRelateProgress(0, 50, 100);
            this.setDescription(_t('Loading events...'));
            return;
        }
        
        if (percent > 0) {
            this.setRelateProgress(percent, 50, 100);
        }
        if (timeleft<0) timeleft = 0;
        timeleft = App.utility.NamesFormatter.formatSeconds(Math.round(timeleft));
        
        this.setDescription(sprintf(_t('Loading events... (remaining ~ %(timeleft)s)'), { timeleft : timeleft }));
    },

    markTerminating: function() {
        this.description = _t('Terminating');
        this.setDescription(this.description);
    },
    markTerminated: function() {
        this.description = _t('Terminated');
        this.setDescription(this.description);
    },
    
    onCancelButtonClick: function(btn) {
        btn.disable();
        this.fireEvent('сancelButtonClick');
    }
} );
