Ext.define('App.view.filter.PeriodEditor', {
    extend: 'App.view.filter.Editor',
    width: 450,
    onRenderContent: function(state) {
        var fromDate = state['fromDate'] && state['fromTime'] ? Ext.Date.parse(state['fromDate'] +' '+ state['fromTime'], "Y-m-d H:i:s") : null;
        var toDate = state['toDate'] && state['toTime'] ? Ext.Date.parse(state['toDate'] +' '+ state['toTime'], "Y-m-d H:i:s") : null;

        var types = Ext.create('Ext.data.Store', {
            fields: ['value', 'name'],
            data : [
                { value: App.utility.Date.TODAY, name: _t('Today') },
                { value: App.utility.Date.YESTERDAY, name: _t('Yesterday') },
                { value: App.utility.Date.WEEK, name: _t('This week') },
                { value: App.utility.Date.LAST_2_DAYS, name: _t('Last 2 days') },
                { value: App.utility.Date.LAST_7_DAYS, name: _t('Last 7 days') },
                { value: App.utility.Date.PREV_WEEK, name: _t('Previous week') },
                { value: App.utility.Date.LAST_2_WEEKS, name: _t('Last 2 weeks') },
                { value: App.utility.Date.MONTH, name: _t('This month') },
                { value: App.utility.Date.LAST_30_DAYS, name: _t('Last 30 days') },
                { value: App.utility.Date.PREV_MONTH, name: _t('Previous month') },
                { value: App.utility.Date.LAST_60_DAYS, name: _t('Last 60 days') },
                { value: App.utility.Date.LAST_3_MONTH, name: _t('Last 3 month') },
                { value: App.utility.Date.LAST_6_MONTH, name: _t('Last 6 month') },
                { value: App.utility.Date.LAST_12_MONTH, name: _t('Last 12 month') },
                { value: App.utility.Date.YEAR, name: _t('This year') },
                { value: App.utility.Date.PREV_YEAR, name: _t('Previous year') },
                { value: App.utility.Date.CUSTOM, name: _t('Related to a Time interval') }
            ]
        });

        this.typesField = Ext.create('Ext.form.ComboBox', {
            store: types,
            anchor: '100%',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'value',
            value: state['type'],
            editable: false,
            listeners: {
                'specialkey': {
                    fn: function(el, e) {
                        if(!el.isExpanded && e.ESC == e.keyCode) {
                            this.hide();
                        }
                    },
                    scope: this
                }
            }
        });

        this.fromDateField = Ext.create('Ext.form.field.Date', {
            margin: '0 2 0 0',
            format: User.getDateFormat(),
            flex: 1,
            value: fromDate ? fromDate : null
        });

        this.fromTimeField = Ext.create('Ext.form.field.Time', {
            format: User.getTimeFormat(),
            width: 75,
            increment: 30,
            value: fromDate ? fromDate : null
        });

        this.toDateField = Ext.create('Ext.form.field.Date', {
            margin: '0 2 0 0',
            format: User.getDateFormat(),
            flex: 1,
            value: toDate ? toDate : null
        });

        this.toTimeField = Ext.create('Ext.form.field.Time', {
            format: User.getTimeFormat(),
            width: 75,
            increment: 30,
            value: toDate ? toDate : null
        });

        this.containerRange = Ext.create('Ext.container.Container', {
            anchor: '100%',
            layout: 'hbox',
            hidden: state['type'] == App.utility.Date.CUSTOM ? false : true,
            hideMode: 'display',
            items : [
                {
                    xtype: 'fieldcontainer',
                    flex: 0.5,
                    fieldLabel: _t('From'),
                    labelAlign: 'top',
                    layout: 'hbox',
                    items: [
                        this.fromDateField,
                        this.fromTimeField
                    ]
                },
                {
                    xtype: 'container',
                    width: 20
                },
                {
                    xtype: 'fieldcontainer',
                    flex: 0.5,
                    fieldLabel: _t('To'),
                    labelAlign: 'top',
                    layout: 'hbox',
                    items: [
                        this.toDateField,
                        this.toTimeField
                    ]
                }
            ]
        });

        this.containerEl = Ext.create('Ext.container.Container', {
            renderTo: this.formEl,
            layout: 'anchor',
            items: [
                {
                    xtype: 'fieldcontainer',
                    anchor: '100%',
                    fieldLabel: _t('Period'),
                    labelAlign: 'left',
                    labelWidth: 60,
                    layout: 'anchor',
                    items: [
                        this.typesField,
                        this.containerRange
                    ]
                }
            ]
        });

        this.typesField.on('change', this.onTypesFieldChange, this);
        this.fromDateField.on('change', this.onFromDateFieldChange, this);
        this.fromTimeField.on('change', this.onFromTimeFieldChange, this);
        this.toDateField.on('change', this.onToDateFieldChange, this);
        this.toTimeField.on('change', this.onToTimeFieldChange, this);

        this.callParent(arguments);
    },

    onTypesFieldChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }

        if (value == App.utility.Date.CUSTOM) {
            this.containerRange.show();
            var toDate = App.storage.Synchronizer.getBackendMaxDate() || Ext.Date.format(new Date(), "Y-m-d H:i:s");
                toDate = Ext.Date.parse(toDate, 'Y-m-d H:i:s');
            var fromDate = new Date(toDate);
                fromDate.setDate(toDate.getDate() - 14);
            var fromDateAbs = Ext.Date.parse(Ext.Date.format(fromDate, "Y-m-d 00:00:00"), "Y-m-d H:i:s");

            this.fromDateField.setValue(fromDateAbs);
            this.fromTimeField.setValue(fromDateAbs);
            this.toDateField.setValue(toDate);
            this.toTimeField.setValue(toDate);

            this.state['type'] = value;
            this.state['fromDate'] = Ext.Date.format(fromDateAbs, 'Y-m-d');
            this.state['fromTime'] = Ext.Date.format(fromDateAbs, 'H:i:s');
            this.state['toDate'] = Ext.Date.format(toDate, 'Y-m-d');
            this.state['toTime'] = Ext.Date.format(toDate, 'H:i:s');

            this.fireEvent('change', this.state);
            return;
        }

        this.state['type'] = value;
        this.state['fromDate'] = null;
        this.state['fromTime'] = null;
        this.state['toDate'] = null;
        this.state['toTime'] = null;
        this.fireEvent('change', this.state);
        this.containerRange.hide();
    },
    onFromDateFieldChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }

        if( value ) {
            this.fromTimeField.setDisabled(false);

            try {
                this.state['fromDate'] = Ext.Date.format(value, 'Y-m-d');
            } catch (e) {
                this.state['fromDate'] = null;
                this.fromTimeField.setDisabled(true);
            }
            this.fireEvent('change', this.state);
        } else {
            this.fromTimeField.setValue(null);
            this.fromTimeField.setDisabled(true);
        }
    },
    onFromTimeFieldChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }

        try {
            this.state['fromTime'] = Ext.Date.format(value, 'H:i:s');
        } catch (e) {
            this.state['fromTime'] = null;
        }

        this.fireEvent('change', this.state);
    },
    onToDateFieldChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }

        if( value ) {
            this.toTimeField.setDisabled(false);

            try {
                this.state['toDate'] = Ext.Date.format(value, 'Y-m-d');
            } catch (e) {
                this.state['toDate'] = null;
                this.toTimeField.setDisabled(true);
            }
            this.fireEvent('change', this.state);
        } else {
            this.toTimeField.setValue(null);
            this.toTimeField.setDisabled(true);
        }
    },
    onToTimeFieldChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }

        try {
            this.state['toTime'] = Ext.Date.format(value, 'H:i:s');
        } catch (e) {
            this.state['toTime'] = null;
        }

        this.fireEvent('change', this.state);
    },
    setState: function(s) {
        this.state = s;
        this.conditionEl.update(this.getDescription(s));

        if (!this.contentRendered) {
            return;
        }

        this.ignoreStateEvents = true;

        var fromDate = s['fromDate'] ? Ext.Date.parse(s['fromDate'] +' '+ s['fromTime'], "Y-m-d H:i:s") : null;
        var fromTime = s['fromTime'] ? Ext.Date.parse(s['fromDate'] +' '+ s['fromTime'], "Y-m-d H:i:s") : null;
        var toDate = s['toDate'] ? Ext.Date.parse(s['toDate'] +' '+ s['toTime'], "Y-m-d H:i:s") : null;
        var toTime = s['toTime'] ? Ext.Date.parse(s['toDate'] +' '+ s['toTime'], "Y-m-d H:i:s") : null;

        this.typesField.setValue(s['type']);

        if (App.utility.Date.CUSTOM == s['type']) {
            this.containerRange.show();
        } else {
            this.containerRange.hide();
        }

        this.fromDateField.setValue(fromDate);
        this.fromTimeField.setValue(fromTime);
        this.toDateField.setValue(toDate);
        this.toTimeField.setValue(toTime);

        this.ignoreStateEvents = false;
    },
    getDefaultState: function() {
        var s = {
            'type': parseInt(User['period']) || 3,
            'fromDate': null,
            'fromTime': null,
            'toDate': null,
            'toTime': null
        }

        return s;
    },

    /**
     * Get's an array of From Date and To Date
     *
     */
    getPeriodDescription: function(state) {
        var fromDate = false;
        var toDate = false;

        fromDate = Ext.Date.parse(state['fromDate'] +' '+ state['fromTime'], "Y-m-d H:i:s");
        fromDate = Ext.util.Format.date(fromDate, User.getDateTimeFormat());
        toDate = Ext.Date.parse(state['toDate'] +' '+ state['toTime'], "Y-m-d H:i:s");
        toDate = Ext.util.Format.date(toDate, User.getDateTimeFormat());

        return sprintf(_t('From_%(from)s_to_%(to)s'), { from : fromDate, to : toDate });
    },

    getTypeDescription: function(state) {
        switch(state['type']) {
            case App.utility.Date.TODAY:
                return _t('Today');
                break;
            case App.utility.Date.YESTERDAY:
                return _t('Yesterday');
                break;
            case App.utility.Date.WEEK:
                return _t('This week');
                break;
            case App.utility.Date.LAST_2_DAYS:
                return _t('Last 2 days');
                break;
            case App.utility.Date.LAST_7_DAYS:
                return _t('Last 7 days');
                break;
            case App.utility.Date.PREV_WEEK:
                return _t('Previous week');
                break;
            case App.utility.Date.LAST_2_WEEKS:
                return _t('Last 2 weeks');
                break;
            case App.utility.Date.MONTH:
                return _t('This month');
                break;
            case App.utility.Date.LAST_30_DAYS:
                return _t('Last 30 days');
                break;
            case App.utility.Date.PREV_MONTH:
                return _t('Previous month');
                break;
            case App.utility.Date.LAST_60_DAYS:
                return _t('Last 60 days');
                break;
            case App.utility.Date.LAST_3_MONTH:
                return _t('Last 3 month');
                break;
            case App.utility.Date.LAST_6_MONTH:
                return _t('Last 6 month');
                break;
            case App.utility.Date.LAST_12_MONTH:
                return _t('Last 12 month');
                break;
            case App.utility.Date.YEAR:
                return _t('This year');
                break;
            case App.utility.Date.PREV_YEAR:
                return _t('Previous year');
                break;
        }
    },

    getDescription: function(s) {
        if (s['type'] == App.utility.Date.CUSTOM) {
            return sprintf('%(periodDesc)s', { periodDesc : this.getPeriodDescription(s)});
        } else {
            return sprintf('%(typeDesc)s', { typeDesc : this.getTypeDescription(s) });
        }
    },

    getFieldDescription: function(s) {
        if (s['type'] == App.utility.Date.CUSTOM) {
            return sprintf('%(periodDesc)s', { periodDesc : this.getPeriodDescription(s) });
        } else {
            return sprintf('%(typeDesc)s', { typeDesc : this.getTypeDescription(s) });
        }
    },

    getFilterDescription: function() {
        var s = this.state;

        if (s['type'] == App.utility.Date.CUSTOM) {
            return _t('Period')+': '+ this.getPeriodDescription(s);
        } else {
            return _t('Period')+': '+ this.getTypeDescription(s);
        }

        return null;
    }
} );
