Ext.define('App.view.filter.TagsEditor', {
    extend: 'App.view.filter.Editor',
    width: 400,
    onRenderContent: function(state) {
        this.tagsField = Ext.create('App.view.filter.field.Tag', {
            fieldLabel: _t('Tags'),
            labelAlign: 'left',
            labelWidth: 100,
            arrayValue: state['tags'],
            anchor: '100%'
        });
        
        this.containerEl = Ext.create('Ext.container.Container', {
            renderTo: this.formEl,
            layout: 'anchor',
            items: [
                this.tagsField
            ]
        });
        
        this.tagsField.on('change', this.onTagsFieldChange, this);
        this.callParent(arguments);
    },
    onTagsFieldChange: function(field) {
        if (this.ignoreStateEvents) {
            return;
        }
        
        this.state['tags'] = field.getArrayValue();
        this.fireEvent('change', this.state);
    },
    hide: function(){
        if(this.tagsField.selectedView != undefined && this.tagsField.selectedView.isVisible()) {
            this.tagsField.hidePicker();
            return;
        } else {
            this.callParent();
        }
    },
    setState: function(s) {
        this.state = s;
        this.conditionEl.update(this.getDescription(s));
        
        if (!this.contentRendered) {
            return;
        }
        
        this.ignoreStateEvents = true;
        this.tagsField.setArrayValue(s['tags']);
        this.ignoreStateEvents = false;
    },
    getDefaultState: function() {
        var s = {
            'tags': new Array()
        }
        
        return s;
    },
    
    getTagsDescription: function(tags) {
        var i, result = new Array();
        
        if( tags.length == 0 ) {
            return undefined;
        }
        
        for( i=0; i < tags.length; i++ ) {
            result.push(tags[i]['name']);
        }
        
        if ( result.length > 0 ) {
            return result.join(', ');
        } else {
            return undefined;
        }      
    },
    
    getDescription: function(state) {
        if (state) {
            var desc = this.getTagsDescription(state['tags']);

            if (desc) {
                return sprintf('%(tagDesc)s', { tagDesc : desc });
            }
        }

        return this.getAnyDescription();
    },
    
    getFieldDescription: function(state) {
	if (state) {
            var desc = this.getTagsDescription(state['tags']);

            if (desc) {
                return sprintf('%(tagDesc)s', { tagDesc : desc });
            }
        }
        
        return this.getAnyFieldDescription();
    },
    
    getFilterDescription: function() {
        var desc = this.getTagsDescription(this.state['tags']);
        
        if (desc) {
            return _t('Tags')+': '+desc;
        }
        
        return null;
    }
});
