Ext.define('App.view.filter.field.Combo', {
    extend: 'Ext.form.field.Display',
    cls: 'w-dst',
    arrayValue: new Array(),
    initComponent: function() {
        this.addEvents('pickerShow', 'pickerHide');
        this.enableBubble('pickerShow', 'pickerHide');

        this.value = this.getPlainValue(this.arrayValue);
        this.callParent(arguments);
    },
    afterRender: function() {
        this.callParent(arguments);
        this.bodyEl = this.inputEl.parent();
        this.inputEl.setVisibilityMode(Ext.Element.DISPLAY);
        this.bodyEl.on('click', this.onClick, this);
    },
    onClick: function(e) {
        if(!this.isDisabled())
            this.showPicker();
    },
    onSelectedViewItemAdd: function() {
        this.picker.alignPicker();
    },
    onSelectedViewItemRemove: function() {
        this.picker.alignPicker();
    },
    onSelectedViewItemClick: function(view, record, item, index, e) {
        var target = Ext.get(e.target);

        if (target.hasCls('close')) {
            this.selectedStore.remove(record);
        }
        
        e.stopEvent();
    },  
    onCollapseWheel: function(e) {
        if (!this.isDestroyed && !e.within(this.bodyEl, false, true) && !e.within(this.picker.getEl(), false, true)) {
            this.hidePicker();
        }
    },
    showPicker: function() {
        if (!this.selectedStore) {
            this.selectedStore = this.createSelectedStore(this.arrayValue);
        }
        
        if (!this.selectedView) {
            this.selectedView = this.createSelectedView(this.bodyEl, this.selectedStore);
            this.selectedView.on('itemclick', this.onSelectedViewItemClick, this);
            this.selectedView.on('itemadd', this.onSelectedViewItemAdd, this);
            this.selectedView.on('itemremove', this.onSelectedViewItemRemove, this);
            this.selectedView.refresh();
        }
        
        if (!this.store) {
            this.store = this.createStore();
        }
        
        if (!this.picker) {
            this.picker = this.createPicker(this.bodyEl, this.store, this.selectedStore, this.selectedView);
        }
        
        this.inputEl.hide();
        this.selectedView.show();
        this.fireEvent('pickerShow', this);
        if (!this.pickerExanded) {
            this.bodyEl.addCls('x-form-display-field-active');
            
            this.picker.expand();
            this.pickerExanded = true;

            this.mon(Ext.getDoc(), {
                mousedown: this.onCollapseWheel,
                scope: this
            });
        }
    },
    hidePicker: function() {
        this.bodyEl.removeCls('x-form-display-field-active');
        
        this.inputEl.show();
        this.selectedView.hide();
        this.fireEvent('pickerHide', this);
        this.setValue(this.getPlainValue(this.getArrayValue()));
        this.picker.collapse();
        this.pickerExanded = false;

        Ext.getDoc().un('mousedown', this.collapseWheel, this);
        
        this.selectedView.un('itemadd', this.onSelectedViewAdd, this);
        this.selectedView.un('itemremove', this.onSelectedViewRemove, this);
    },
    setArrayValue: function(destinations) {
        this.setValue(this.getPlainValue(destinations));
        
        if (!this.selectedStore) {
            this.arrayValue = destinations;
        } else {
            this.selectedStore.removeAll();
        
            if (Ext.isArray(destinations)) {
                this.selectedStore.add(Ext.clone(destinations));
            }
        }
    },
    getArrayValue: function() {
        var result = new Array(), i, range;
        
        if (!this.selectedStore) {
            return this.arrayValue;
        }
        
        if (this.selectedStore) {
            range = this.selectedStore.getRange();
            
            for (i = 0; i < range.length; i++) {
                result.push(range[i].data);
            }
        }
        
        return result;
    },
    getPlainValue: function(range) {
        var i, plain = new Array();

        if (range) {
            for (i=0; i < range.length; i++) {
                plain.push(range[i]['name']);
            }

            if (plain.length > 0) {
                return plain.join(', ');
            }
        }
        
        return _t('Any')
    }
} );