Ext.define('App.view.grouping.Picker', {
    extend: 'Ext.view.View',
    hidden: true,
    hideMode: 'display',
    floating: true,
    cls: 'w-viewport-picker',
    dirty: false,
    initComponent: function() {
        this.groupColumns = [
            { id: '1', name: 'HOUR', text: _t('Hour'), group: 'default' },
            { id: '1', name: 'HOUR_OF_DAY', text: _t('Hour of day'), group: 'default' },
            { id: '2', name: 'DAY', text: _t('Day'), group: 'default' },
            { id: '3', name: 'DAY_OF_WEEK', text: _t('Day of week'), group: 'default' },
            { id: '16',name: 'WEEK', text: _t('Week'), group: 'default' },
            { id: '4', name: 'MONTH', text: _t('Month'), group: 'default' },
            { id: '4', name: 'MONTH_OF_YEAR', text: _t('Month of year'), group: 'default' },
            { id: '5', name: 'YEAR', text: _t('Year'), group: 'default' },
            { id: '6', name: 'USER', text: _t('User'), group: 'default' },
            { id: '7', name: 'GROUP', text: _t('Group'), group: 'default' },
            { id: '8', name: 'CLIENT', text: _t('Business partner'), group: 'default' },
            { id: '9', name: 'COMPANY', text: _t('Company'), group: 'default' },
            { id: '10', name: 'NUMBER', text: _t('Number'), group: 'default' },
            { id: '11', name: 'TRUNK', text: _t('Trunk'), group: 'default' },
            { id: '16', name: 'TAG', text: _t('Tag'), group: 'default'},
            { id: '12', name: 'COUNTRY', text: _t('Country'), group: 'default' },
            { id: '13', name: 'CLASS', text: _t('Class'), group: 'default' },
            { id: '14', name: 'USERS', text: _t('Users communication'), group: 'default' },
            { id: '15', name: 'GROUPS', text: _t('Groups communication'), group: 'default' }
        ];

        this.addEvents('expand', 'collapse', 'change');

        this.store = Ext.create('Ext.data.JsonStore', {
            fields: [
                { name: 'id' },
                { name: 'name' },
                { name: 'text' },
                { name: 'group' },
                { name: 'active', defaultValue: false },
                { name: 'enable', defaultValue: true },
                { name: 'hidden', defaultValue: false }
            ],
            data: this.groupColumns
        });

        this.tpl = new Ext.XTemplate(
            '<div class="w-viewport-picker-tl"><div class="w-viewport-picker-tr"><div class="w-viewport-picker-tt">',
                '<div class="w-viewport-picker-arrow">',
                    _t('Grouping by'),
                '</div>',
            '</div></div></div>',
            '<div class="w-viewport-picker-cl"><div class="w-viewport-picker-cr">',
                '<div class="x-box-scroller-top w-viewport-picker-list-scroll-top">',
                    '<div class="x-menu-scroll-top"></div>',
                '</div>',
                '<div class="w-viewport-picker-list">',
                    '<div class="w-viewport-picker-list-scroll">',
                    '<div class="w-viewport-picker-list-scroll-wrap">',
                    '<tpl for=".">',
                        '<div class="item <tpl if="hidden == true">x-hidden</tpl> <tpl if="permanent == true">item-permanent</tpl> <tpl if="enable == false">x-menu-item-disabled</tpl>">',
                            '<div class="item-left"><div class="item-right">',
                                '<span class="item-name">{text}</span>',
                            '</div></div>',
                        '</div>',
                    '</tpl>',
                    '</div>',
                    '</div>',
                '</div>',
                '<div class="x-box-scroller-bottom w-viewport-picker-list-scroll-bottom">',
                    '<div class="x-menu-scroll-bottom"></div>',
                '</div>',
                '<div class="w-viewer-table-columns-clear"><a class="w-viewer-table-columns-clear-text">',
                    _t('Reset'),
                '</a></div>',
            '</div></div>',
            '<div class="w-viewport-picker-bl"><div class="w-viewport-picker-br"><div class="w-viewport-picker-bb"></div></div></div>'
        );

        this.itemSelector = 'div.item';
        this.callParent(arguments);
        this.bind = true;
    },

    setState: function(state) {
        this.store.each(function(rec) {
            if(state.indexOf(rec.get('name')) != -1 ){
                rec.set('active', true);
                rec.set('enable', true);
            } else {
                rec.set('active', false);
                rec.set('enable', true);
            }
        });
    },

    getState: function() {
        var results = new Array(), columns;

        columns = this.store.queryBy(function(rec) {
            return rec.get('active') == true && rec.get('enable') == true;
        });
        columns.each(function(rec) {
            results.push(rec.get('name'));
        });

        return results;
    },

    getFirstStateName: function() {
        var results = new Array(), columns;

        columns = this.store.queryBy(function(rec) {
            return rec.get('active') == true && rec.get('enable') == true;
        });
        columns.each(function(rec) {
            results.push(rec.get('text'));
        });

        return results[0]||'';
    },

    onHeadElClick: function() {
        this.hide();
    },

    onResetElClick: function() {
        var i, r;
        var len = this.store.getCount();

        for (i=0; i < len; i++) {
            r = this.store.getAt(i);

            if (r.get('active') || !r.get('enable')) {
                r.set('active', false);
                r.set('enable', true);
            }
        }

        this.hide();
    },

    bindView: function() {
        this.headEl = this.getEl().down('div.w-viewport-picker-arrow');
        this.headEl.on('click', this.onHeadElClick, this);

        this.resetEl = this.getEl().down('a.w-viewer-table-columns-clear-text');
        this.resetEl.on('click', this.onResetElClick, this);

        this.listEl = this.getEl().down('div.w-viewport-picker-list');
        this.listScrollTopEl = this.getEl().down('div.w-viewport-picker-list-scroll-top');
        this.listScrollTopEl.setVisibilityMode(Ext.core.Element.DISPLAY);
        this.listScrollTopEl.on('click', this.onListScrollTopElClick, this);
        this.listScrollBottomEl = this.getEl().down('div.w-viewport-picker-list-scroll-bottom');
        this.listScrollBottomEl.setVisibilityMode(Ext.core.Element.DISPLAY);
        this.listScrollBottomEl.on('click', this.onListScrollBottomElClick, this);
        this.listScrollEl = this.getEl().down('div.w-viewport-picker-list-scroll');
        this.listScrollWrapEl = this.getEl().down('div.w-viewport-picker-list-scroll-wrap');
    },

    onListScrollTopElClick: function() {
        this.listScrollEl.scroll('top', 26, true)
    },

    onListScrollBottomElClick: function() {
        this.listScrollEl.scroll('bottom', 26, true)
    },


    show: function() {
        // Show component
        if (this.isVisible()) {
            return;
        }

        Ext.getDoc().on('mousedown', this.collapseWheel, this);
        Ext.EventManager.onWindowResize(this.align, this);

        this.callParent(arguments);

        if (this.bind)  {
            this.bindView();
            this.bind = false;
        }

        this.shownState = this.getState();
        this.align();

        if(!this.KeyMapOnEsc) {
            this.KeyMapOnEsc = new Ext.util.KeyMap(Ext.getBody(), [{
               key: Ext.EventObject.ESC,
               defaultEventAction: 'preventDefault',
               scope: this,
               fn: this.hide
            }]);
        } else {
            this.KeyMapOnEsc.enable();
        }

    },
    hide: function() {
        Ext.getDoc().un('mousedown', this.collapseWheel, this);
        Ext.EventManager.removeResizeListener(this.align, this);

        this.callParent(arguments);

        // Calculating changes
        var i, missmatch = false;

        this.currentState = this.getState();

        if (this.shownState.length != this.currentState.length) {
            missmatch = true;
        }

        if (!missmatch) {
            for (i=0; i < this.currentState.length; i++) {
                if (this.shownState[i] != this.currentState[i]) {
                    missmatch = true;
                    break;
                }
            }
        }

        if (missmatch) {
            this.fireEvent('change', this.currentState);
        }
        this.KeyMapOnEsc.disable();
    },
    align: function() {
        var trgLeft = this.target.getLeft();
            trgLeft = trgLeft - 10;
        var trgTop = this.target.getTop();
            trgTop = trgTop - 5;
        var trgWidth = this.target.getWidth();

        if (trgWidth < 180) {
            trgWidth = 180;
        }

        this.listEl.setHeight('auto');
        this.listScrollEl.setHeight('auto');

        var offset = Ext.getBody().getHeight() - (trgTop + this.getHeight());
        var height = Ext.getBody().getHeight() - trgTop;

        if (offset < 0) {
            var offsetBt = this.getHeight() - this.listEl.getHeight();
            var heightList = height - offsetBt;

            this.listScrollTopEl.show();
            this.listScrollBottomEl.show();

            this.listEl.setHeight(heightList - this.listScrollTopEl.getHeight() - this.listScrollBottomEl.getHeight());
            this.listScrollEl.setHeight(this.listEl.getHeight());
            this.listScrollEl.setWidth(trgWidth + 4 + 20);
            this.listScrollWrapEl.setWidth(trgWidth - 4);
        } else {
            this.listScrollTopEl.hide();
            this.listScrollBottomEl.hide();
        }

        this.setWidth(trgWidth);
        this.setPosition(trgLeft, trgTop, false);
    },
    collapseWheel: function(event, element) {
        if (!event.within(this.el, false, true)) {
            this.hide();
        }
    },
    onItemClick: function(record, item, index, event) {
        if (!record.get('active')) {
            var group = record.get('group');
            var i, r;
            var len = this.store.getCount();

            for (i=0; i < len; i++) {
                r = this.store.getAt(i);

                if (r.get('group') == group && r.get('active')) {
                    r.set('active', false);
                    break;
                }
            }

            record.set('active', true);
        }

        this.hide();
    },
    clearGroupSelect: function(record){
        var i, r;
        if (record.get('group')) {
            var group_order;

            for (i=0; i < this.store.getCount(); i++) {
                r = this.store.getAt(i);

                if (r.get('group') == record.get('group') && r.get('active')) {
                    group_order = r.get('order');
                    r.set('order', 0);
                    r.set('active', false);
                }
            }
            if(group_order) {
                record.set('order', group_order);
                record.set('active', true);
                return true;
            }
        }
        return false;
    }
});
