Ext.define('App.view.viewer.Sync', {
    extend: 'Ext.window.Window',
    title: '',
    width: 360,
    height: 95,
    modal: true,
    resizable: false,
    closable: false,
    bodyStyle: 'background: transparent; border: 0;',
    constructor: function() {
        this.addEvents('complete', 'terminated');
        this.callParent(arguments);
    },
    initComponent: function() {
        this.synchronizer = App.storage.Synchronizer;
        
        this.descriptionCt = Ext.create('Ext.container.Container', {
            cls: 'w-sync-description'
        });
        
        this.progressActiveCt = Ext.create('Ext.container.Container', {
            cls: 'w-boot-progress-active',
            width: 0
        });
        
        this.description = _t('Remaining time: Unknown');
        this.progressDescriptionCt = Ext.create('Ext.container.Container', {
            cls: 'w-sync-progress-description',
            html: this.description
        });
        
        this.progressCt = Ext.create('Ext.container.Container', {
            cls: 'w-sync-progress',
            items: [
                this.progressActiveCt
            ]
        });
        
        this.progressContainerCt = Ext.create('Ext.container.Container', {
            layout: 'hbox',
            margin: '5 10 0 10',
            items: [
                {
                    xtype: 'container',
                    width: 330,
                    items: [
                        this.progressCt,
                        this.progressDescriptionCt
                    ]
                }
            ]
        });
        
        this.cancelButton = Ext.create('Ext.button.Button', {
            text: _t('Cancel')
        });
        
        this.items = [
            this.progressContainerCt
        ]

        this.buttonAlign = 'center';

        this.buttons = [
            this.cancelButton
        ]
        
        this.cancelButton.on('click', this.onCancelButtonClick, this);
        this.callParent(arguments);
    },
    setProgress: function(percent) {
        var width = (330 / 100) * percent;
        var animation = {
            to: {
                width: width
            }
        }
        
        this.progressActiveCt.stopAnimation();
        this.progressActiveCt.animate(animation);
    },
    setDescription: function(text) {
        this.progressDescriptionCt.getEl().update(text);
    },
    bindSynchronizer: function() {
        this.synchronizer.on('syncEventsChange', this.onSyncEventsChange, this);
        this.synchronizer.on('syncEventsComplete', this.onSyncEventsComplete, this);
        this.synchronizer.on('syncTerminated', this.onSyncTerminated, this);
    },
    unbindSynchronizer: function() {
        this.synchronizer.un('syncEventsChange', this.onSyncEventsChange, this);
        this.synchronizer.un('syncEventsComplete', this.onSyncEventsComplete, this);
        this.synchronizer.un('syncTerminated', this.onSyncTerminated, this);
    },
    sync: function(dateFrom, dateTo) {
        dateFrom = dateFrom.substr(0, 10);
        dateTo = dateTo.substr(0, 10);

        this.processTimer();

        if (this.synchronizer.isWorking()) {
            this.workDateFrom = dateFrom;
            this.workDateTo = dateTo;
            this.synchronizer.on('syncTerminated', this.onSyncWorkTerminated, this);
            this.synchronizer.fireEvent('syncTerminate');
            return;
        }

        this.bindSynchronizer();
        this.synchronizer.fireEvent('syncEvents', dateFrom, dateTo);
    },
    onSyncWorkTerminated: function() {
        console.log('sync', 'onSyncWorkTerminated');

        this.sync(this.workDateFrom, this.workDateTo);
    },
    onSyncEventsChange: function(percent, timeleft) {
        if (!percent && !timeleft) {
            this.setRelateProgress(0, 24);
            this.description = _t('Remaining time: Unknown');
            this.setDescription(this.description);
            return;
        }
        
        if (percent > 0) {
            this.setProgress(percent);
        }
        
        timeleft = App.utility.NamesFormatter.formatSeconds(Math.round(timeleft));
        
        this.description = sprintf(_t('Remaining time: ~ %(timeleft)s'), { timeleft : timeleft });
        this.setDescription(this.description);
    },
    onSyncEventsComplete: function() {
        this.terminateTimer();
        this.unbindSynchronizer();
        this.fireEvent('complete');
        this.close();
    },
    onCancelButtonClick: function(btn) {
        btn.disable();
        //terminated
        this.description = _t('Terminating');
        this.setDescription(this.description);
        
        this.synchronizer.fireEvent('syncTerminate');
    },
    onSyncTerminated: function() {
        this.terminateTimer();
        this.unbindSynchronizer();
        this.fireEvent('terminated');
        this.close();
    },
    processTimer: function() {
        var self = this;
        
        if (this.timerTerminate) {
            return;
        }
        
        if (!this.timerTurn) {
            this.timerTurn = 0;
        }
        
        if (!this.timerText) {
            this.timerText = '';
        }
        
        if (this.timerTurn == 5) {
            this.timerTurn = 0;
            this.timerText = '';
        }
        
        this.timerTurn++;
        this.timerText += '.';
        this.setDescription( this.description + this.timerText );
        
        setTimeout(function() { self.processTimer.call(self) }, 500);
    },
    terminateTimer: function() {
        this.timerTerminate = true;
    }
});
