Ext.define('App.view.viewer.table.Chunker', {
    singleton: true,
    requires: ['Ext.XTemplate'],
    metaTableTpl: [
        '{[this.openTableWrap()]}',
        '<table class="' + Ext.baseCSSPrefix + 'grid-table ' + Ext.baseCSSPrefix + 'grid-table-resizer" border="0" cellspacing="0" cellpadding="0" {[this.embedFullWidth()]}>',
            '<tbody>',
            '<tr class="' + Ext.baseCSSPrefix + 'grid-header-row">',
            '<tpl for="columns">',
                '<th class="' + Ext.baseCSSPrefix + 'grid-col-resizer-{id}" style="width: {width}px; height: 0px;"></th>',
            '</tpl>',
            '</tr>',
            '{[this.openRows()]}',
                '{row}',
                '<tpl for="features">',
                    '{[this.embedFeature(values, parent, xindex, xcount)]}',
                '</tpl>',
            '{[this.closeRows()]}',
            '</tbody>',
        '</table>',
        '{[this.closeTableWrap()]}'
    ],

    constructor: function() {
        Ext.XTemplate.prototype.recurse = function(values, reference) {
            return this.apply(reference ? values[reference] : values);
        };
    },

    embedFeature: function(values, parent, x, xcount) {
        var tpl = '';
        if (!values.disabled) {
            tpl = values.getFeatureTpl(values, parent, x, xcount);
        }
        return tpl;
    },

    embedFullWidth: function() {
        return 'style="width: {fullWidth}px;"';
    },

    openRows: function() {
        return '<tpl for="rows">';
    },

    closeRows: function() {
        return '</tpl>';
    },

    metaRowTpl: [
        '<tr class="' + Ext.baseCSSPrefix + 'grid-row {addlSelector} {[this.embedRowCls()]}" {[this.embedRowAttr()]}>',
            '<tpl for="columns">',
                '<td class="{cls} ' + Ext.baseCSSPrefix + 'grid-cell ' + Ext.baseCSSPrefix + 'grid-cell-{columnId} {{id}-modified} {{id}-tdCls} {[this.firstOrLastCls(xindex, xcount)]}" {{id}-tdAttr}><div unselectable="on" class="' + Ext.baseCSSPrefix + 'grid-cell-inner ' + Ext.baseCSSPrefix + 'unselectable" style="{{id}-style}; text-align: {align};">{{id}}</div></td>',
            '</tpl>',
        '</tr>'
    ],

    firstOrLastCls: function(xindex, xcount) {
        var cssCls = '';
        if (xindex === 1) {
            cssCls = Ext.baseCSSPrefix + 'grid-cell-first';
        } else if (xindex === xcount) {
            cssCls = Ext.baseCSSPrefix + 'grid-cell-last';
        }
        return cssCls;
    },

    embedRowCls: function() {
        return '{rowCls}';
    },

    embedRowAttr: function() {
        return '{rowAttr}';
    },

    openTableWrap: function() {
        return '';
    },

    closeTableWrap: function() {
        return '';
    },

    getTableTpl: function(cfg) {
        // Remove hidden columns
        var columns = new Array();
        var columnsOrig = cfg.columns;
        var columnsItems = cfg.columnsItems;
        var columnsGrouped = new Array();
        var tpl, headerTpl, rowTpl, i, column, columnOrig;

        for (i=0; i < columnsItems.length; i++) {
            column = columnsItems.get(i);
            columnOrig = columnsOrig[i];

            if (column['hidden']) {
                continue;
            }
            if (column['grouped']) {
                columnOrig['grouped'] = true;
                columnsGrouped.push(columns[i]);
            }

            columns.push(columnOrig);
        }


        headerTpl = '<tr class="x-grid-header-row">';
        for (i=0; i < columns.length; i++) {
            headerTpl += '<th class="x-grid-col-resizer-'+ columns[i]['columnId'] +'" style="width: '+ columns[i]['width'] +'px; height: 0px;"></th>';
        }
        headerTpl += '</tr>';

        rowTpl = '<tr class="x-grid-row">';
        for (i=0; i < columns.length; i++) {
            if (columns[i]['grouped']) {
                rowTpl += '<tpl if="values[\''+ columns[i]['columnId'] +'-rowspan\'] != 0">';
                rowTpl +=     '<td class="x-grid-cell-grouped" rowspan="{'+ columns[i]['columnId'] +'-rowspan}" {'+ columns[i]['columnId'] +'-tdAttr}>';
                rowTpl +=          '<div unselectable="on" class="x-grid-cell-inner x-unselectable" style="{'+ columns[i]['columnId'] +'-style}; text-align: left;">{'+ columns[i]['columnId'] +'}</div>';
                rowTpl +=     '</td>';
                rowTpl += '</tpl>';
            } else {
                rowTpl += '<td class="x-grid-cell x-grid-cell-'+ columns[i]['columnId'] +' {'+ columns[i]['columnId'] +'-modified} {'+ columns[i]['columnId'] +'-tdCls} " {'+ columns[i]['columnId'] +'-tdAttr}>';
                rowTpl +=     '<div unselectable="on" class="x-grid-cell-inner x-unselectable" style="{'+ columns[i]['columnId'] +'-style}; text-align: left;">{'+ columns[i]['columnId'] +'}</div>';
                rowTpl += '</td>';
            }
        }
        rowTpl += '</tr>';

        tpl = '<table class="x-grid-table x-grid-table-resizer" border="0" cellspacing="0" cellpadding="0" style="width: {fullWidth}px;">' +
                  '<tbody>' +
                      headerTpl +
                      '<tpl for="rows">' +
                          rowTpl +
                      '</tpl>' +
                  '</tbody>' +
              '</table>';

        tpl = Ext.create('Ext.XTemplate', tpl);

        return tpl;
    }
});